'From Squeak2.9alpha of 13 June 2000 [latest update: #2447] on 21 August 2000 at 12:00:03 pm'!
"Change Set:		RemoveErrorHandler
Date:			9 June 2000
Author:			Lex Spoon

Removes the redundant errorHandler instvar of Process.  errorHandler is just a special case of handling Error exceptions"!

Link subclass: #Process
	instanceVariableNames: 'suspendedContext priority myList errorHandler island '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!
ProtoObject makeSelector: #cannotInterpret: privilaged: false!
!ProtoObject methodsFor: 'system primitives' stamp: 'ls 7/10/2000 15:58'!
cannotInterpret: aMessage 
	 "Handle the fact that there was an attempt to send the given message to the receiver but a null methodDictionary was encountered while looking up the message selector.  Hopefully this is the result of encountering a stub for a swapped out class which induces this exception on purpose."

"If this is the result of encountering a swap-out stub, then simulating the lookup in Smalltalk should suffice to install the class properly, and the message may be resent."

	| |
	(self class lookupSelector: aMessage selector) == nil ifFalse:
		["Simulated lookup succeeded -- resend the message."
		^ aMessage sentTo: self].

	"Could not recover by simulated lookup -- it's an error"
	self error: 'MethodDictionary fault'.

	^ aMessage sentTo: self! !

Object makeSelector: #notify: privilaged: false!
!Object methodsFor: 'error handling' stamp: 'ls 6/27/2000 10:27'!
notify: aString 
	"Create and schedule a Notifier with the argument as the message in 
	order to request confirmation before a process can proceed."

	Debugger
		openContext: Debugger myContext
		label: 'Notifier'
		contents: aString

	"nil notify: 'confirmation message'"! !

BlockContext makeSelector: #ifError: privilaged: false!
!BlockContext methodsFor: 'evaluating' stamp: 'ls 6/9/2000 11:40'!
ifError: errorHandlerBlock
	"Evaluate the block represented by the receiver. If an error occurs the given is evaluated with the error message and the receiver as parameters. The error handler block may return a value to be used if the receiver block gets an error. The receiver should not contain an explicit return statement as this would leave an obsolete error handler hanging around."
	"Note: it is more flexible to use #on:do: instead.  #on:do: allows trapping only specific errors, and it allows querying more details on what exactly happened"

	"Examples:
		[1 whatsUpDoc] ifError: [:err :rcvr | ^ 'huh?'].
		[1 / 0] ifError: [:err :rcvr |
			'ZeroDivide' = err
				ifTrue: [^ Float infinity]
				ifFalse: [self error: err]]
"

	^self on: Error do: [:ex |
		^errorHandlerBlock value: ex description value: ex receiver].


! !
BlockContext makeSelector: #value:ifError: privilaged: false!
!BlockContext methodsFor: 'evaluating' stamp: 'ls 6/9/2000 11:41'!
value: arg1 ifError: aBlock
	"Evaluate the block represented by the receiver. If an error occurs aBlock is evaluated
	 with the error message and the receiver as parameters"

	^[ self value: arg1 ] on: Error do: aBlock! !

WorldState makeSelector: #displayWorldSafely: privilaged: false!
!WorldState methodsFor: 'update cycle' stamp: 'ls 6/14/2000 14:15'!
displayWorldSafely: aWorld
	"Update this world's display and keep track of errors during draw methods."

	| errCtx errMorph |

	[aWorld displayWorld] on: Error do: [ :ex |
		"Handle a drawing error"
		errCtx _ Debugger myContext.
		[
			errCtx _ errCtx sender.
			"Search the sender chain to find the morph causing the problem"
			[errCtx notNil and:[(errCtx receiver isKindOf: Morph) not]] 
				whileTrue:[errCtx _ errCtx sender].
			"If we're at the root of the context chain then we have a fatal drawing problem"
			errCtx == nil ifTrue:[^self handleFatalDrawingError: ex messageText].
			errMorph _ errCtx receiver.
			"If the morph causing the problem has already the #drawError flag set,
			then search for the next morph above in the caller chain."
			errMorph hasProperty: #errorOnDraw
		] whileTrue.
		errMorph setProperty: #errorOnDraw toValue: true.
		"Re-raise the error"
		ex pass.
	].! !
WorldState makeSelector: #runStepMethods privilaged: false!
!WorldState methodsFor: 'stepping' stamp: 'ls 6/9/2000 11:50'!
runStepMethods
	"Run morph 'step' methods whose time has come. Purge any morphs that are no longer in this world.
	ar 3/13/1999: Remove buggy morphs from the step list so that they don't raise repeated errors."

	| now deletions wakeupTime morphToStep nextStepTime |
	self stepList size = 0 ifTrue: [^ self].
	now _ Time millisecondClockValue.
	((now < self lastStepTime) or: [(now - self lastStepTime) > 5000])
		 ifTrue: [self adjustWakeupTimes].  "clock slipped"
	deletions _ nil.
	"Note: Put the following into an error handler to prevent errors happening on stepping"
	[self stepList do: [:entry |
		wakeupTime _ entry at: 2.
		morphToStep _ entry at: 1.
		morphToStep world == self
			ifTrue:
				[wakeupTime <= now
					ifTrue:
					["step the morph"
						morphToStep stepAt: now.

						"arrange for it to be stepped again"
						nextStepTime _ wakeupTime + ((now + 1 - wakeupTime) roundUpTo: (morphToStep stepTime max: 1)).

						entry at: 2 put: nextStepTime. ]]
			ifFalse:
				[deletions ifNil: [deletions _ OrderedCollection new].
				deletions addLast: morphToStep]]] 
	on: Error do: [:ex |
		self stopStepping: morphToStep. "Stop this guy right now"
		morphToStep setProperty: #errorOnStep toValue: true. "Remember stepping"
		ex pass ].

	deletions ifNotNil:
		[deletions do: [:deletedM |
			self stopStepping: deletedM.
			deletedM stopStepping]].
	self lastStepTime: now.
! !
WorldState makeSelector: #runStepMethodsIn: privilaged: false!
!WorldState methodsFor: 'stepping' stamp: 'ls 8/16/2000 17:28'!
runStepMethodsIn: aWorld
	"Run morph 'step' methods whose time has come. Purge any morphs that are no longer in this world.
	ar 3/13/1999: Remove buggy morphs from the step list so that they don't raise repeated errors."

	| now deletions wakeupTime morphToStep nextTime |
	stepList size = 0 ifTrue: [^ self].
	now _ Time millisecondClockValue.
	((now < lastStepTime) or: [(now - lastStepTime) > 5000])
		 ifTrue: [self adjustWakeupTimes].  "clock slipped"
	deletions _ nil.
	"Note: Put the following into an error handler to prevent errors happening on stepping"
	[stepList do: [:entry |
		wakeupTime _ entry at: 2.
		morphToStep _ entry at: 1.
		morphToStep world == aWorld
			ifTrue:
				[wakeupTime <= now
					ifTrue:
						[| numStepIntervals stepTime |
						morphToStep stepAt: now.
						stepTime := morphToStep stepTime max: 1.
						numStepIntervals := ((now - wakeupTime) / stepTime)   ceiling.
						nextTime := wakeupTime + (stepTime * numStepIntervals).
						entry at: 2 put: nextTime]]
			ifFalse:
				[deletions ifNil: [deletions _ OrderedCollection new].
				deletions addLast: morphToStep]]]
	 on: Error do:  [:ex |
		self stopStepping: morphToStep. "Stop this guy right now"
		morphToStep setProperty: #errorOnStep toValue: true. "Remember stepping"
		ex pass ].

	deletions ifNotNil:
		[deletions do: [:deletedM |
			self stopStepping: deletedM.
			deletedM stopStepping]].
	lastStepTime _ now! !

Process removeSelector: #errorHandler!
Process removeSelector: #errorHandler:!
Link subclass: #Process
	instanceVariableNames: 'suspendedContext priority myList island '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Processes'!
Object removeSelector: #cannotInterpret:!
